import os
import sys
import time
import logging
import argparse

from _version import ___version___ as version

_DESCRIPTION = """Workspaces null executable, a status capture test of the system. Version {}"""

logger = logging.getLogger("null")
logger.setLevel(logging.DEBUG)

class Null:
    def __init__(self, args, verbose):
        self.args = args
        self.verbose = verbose
        self.args_to_funcs = {
            'greeting': self.print_greeting,
            'exit': self.exit_with_failure,
            'nap': self.take_nap,
            'dump': self.dump_core
        }

    def print_greeting(self):
        logger.debug("Hello, world!")
        if self.verbose:
            logger.debug("And goodbye, world...")

    def exit_with_failure(self):
        if self.verbose:
            logger.error("Error purposefully induced.")
        sys.exit('Exiting with status code -1')

    def take_nap(self):
        print(self.verbose)
        if self.verbose:
            print("wtf??")
            logger.debug("Going to sleep...")
        time.sleep(5)
        if self.verbose:
            logger.debug("Waking up.")

    def dump_core(self):
        if self.verbose:
            logger.debug("Aborting and dumping core...", stack_info=True)
        os.abort()

    def execute(self):
        for arg, val in vars(self.args).items():
            if val and arg in self.args_to_funcs:
                self.args_to_funcs[arg]()

def make_arg_parser():
    parser = argparse.ArgumentParser(description=_DESCRIPTION.format(version),
                                     formatter_class=argparse.RawTextHelpFormatter)
    options = parser.add_argument_group('options', 'settings for altered program behavior')
    options.add_argument('-v', '--verbose', action='store_true',
                         required=False, dest='verbose', default=False,
                         help='allow the program the gift of speech')
    functions = parser.add_mutually_exclusive_group(required=False)
    functions.add_argument('-g', '--greeting', action='store_true',
                           required=False, dest='greeting', default=False,
                           help='print out a friendly greeting')
    functions.add_argument('-e', '--exit', action='store_true',
                           required=False, dest='exit', default=False,
                           help='print error message and exit with status code -1')
    functions.add_argument('-n', '--nap', action='store_true',
                           required=False, dest='nap', default=False,
                           help='take a short nap')
    functions.add_argument('-d', '--dump', action='store_true',
                           required=False, dest='dump', default=False,
                           help='abort program and dump core')
    return parser


def main():
    arg_parser = make_arg_parser()
    args = arg_parser.parse_args()
    null = Null(args, args.verbose)
    null.execute()

if __name__ == '__main__':
    main()